--- main/say.c.orig     2009-05-05 17:18:53.000000000 +0600
+++ main/say.c  2009-05-05 18:04:39.000000000 +0600
@@ -29,6 +29,9 @@
  *                                             Next Generation Networks (NGN).
  * \note 2007-03-20 : Support for Thai added by Dome C. <dome@tel.co.th>,
  *                                             IP Crossing Co., Ltd.
+ * \note 2009-05-05 : Support for Urdu Say number added by A.R. Nasir Qureshi <nasir@mitco.pk>,
+ *                                             MITCO.
+ *                    based on work of Atif Rasheed <atif@iphonica.com> (http://lists.digium.com/pipermail/asterisk-dev/2005-July/013933.html)
  */

 #include "asterisk.h"
@@ -349,6 +352,7 @@
 static int ast_say_number_full_ge(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
 static int ast_say_number_full_hu(struct ast_channel *chan, int num, const char *ints, const char *language, int audiofd, int ctrlfd);
 static int ast_say_number_full_th(struct ast_channel *chan, int num, const char *ints, const char *language, int audiofd, int ctrlfd);
+static int ast_say_number_full_ur(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);

 /* Forward declarations of language specific variants of ast_say_enumeration_full */
 static int ast_say_enumeration_full_en(struct ast_channel *chan, int num, const char *ints, const char *language, int audiofd, int ctrlfd);
@@ -445,6 +449,8 @@
           return(ast_say_number_full_es(chan, num, ints, language, options, audiofd, ctrlfd));
        } else if (!strcasecmp(language, "fr") ) {      /* French syntax */
           return(ast_say_number_full_fr(chan, num, ints, language, options, audiofd, ctrlfd));
+       } else if (!strcasecmp(language, "ur") ) {      /* Urdu syntax */
+          return(ast_say_number_full_ur(chan, num, ints, language, options, audiofd, ctrlfd));
        } else if (!strcasecmp(language, "he") ) {      /* Hebrew syntax */
           return(ast_say_number_full_he(chan, num, ints, language, options, audiofd, ctrlfd));
        } else if (!strcasecmp(language, "hu") ) {      /* Hungarian syntax */
@@ -1184,6 +1190,77 @@
        return res;
 }

+/* Urdu, the national language of Pakistan, has different
+ * pronounciation of each number till 100.
+ * And the division of numbers for 100 called Sua (hundred), 1,000 called
+ * hazar (thousand), 100,000 called lac and 100,00,000 called crore.
+ * This requires voice files for each number from 0 to 100 to be placed
+ * in /var/lib/asterisk/sounds/digits/ur/. Addtional files required are
+ * crore for 100,00,000 and lac for 100,000.
+*/
+
+
+static int ast_say_number_full_ur(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd)
+{
+       int res = 0;
+       int playh = 0;
+       char fn[256] = "";
+       if (!num)
+               return ast_say_digits_full(chan, 0,ints, language, audiofd, ctrlfd);
+
+       while(!res && (num || playh)) {
+               if (playh) {
+                       snprintf(fn, sizeof(fn), "digits/hundred");
+                       playh = 0;
+               } else  if (num < 100) {
+                       snprintf(fn, sizeof(fn), "digits/%d", num);
+                       num = 0;
+               } else {
+                       if (num < 1000){
+                               snprintf(fn, sizeof(fn), "digits/%d", (num/100));
+                               playh++;
+                               num -= ((num / 100) * 100);
+                       } else {
+                               if (num < 100000) { /* 1,00,000 */
+                                       res = ast_say_number_full_ur(chan, num / 1000, ints, language, options, audiofd, ctrlfd);
+                                       if (res)
+                                               return res;
+                                       num = num % 1000;
+                                       snprintf(fn, sizeof(fn), "digits/thousand");
+                               } else {
+                                       if (num < 10000000) { /* 1,00,00,000 */
+                                               res = ast_say_number_full_ur(chan, num / 100000, ints, language, options, audiofd, ctrlfd);
+                                               if (res)
+                                                       return res;
+                                               num = num % 100000;
+                                               snprintf(fn, sizeof(fn), "digits/lac");
+                                       } else {
+                                               if (num < 1000000000) { /* 1,00,00,00,000 */
+                                                       res = ast_say_number_full_ur(chan, num / 10000000, ints, language, options, audiofd, ctrlfd);
+                                                       if (res)
+                                                               return res;
+                                                       num = num % 10000000;
+                                                       snprintf(fn, sizeof(fn), "digits/crore");
+                                               } else {
+                                                       ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
+                                                       res = -1;
+                                               }
+                                       }
+                               }
+                       }
+               }
+               if (!res) {
+                       if(!ast_streamfile(chan, fn, language)) {
+                               if ((audiofd > -1) && (ctrlfd > -1))
+                                       res = ast_waitstream_full(chan, ints, audiofd, ctrlfd);
+                               else
+                                       res = ast_waitstream(chan, ints);
+                       }
+                       ast_stopstream(chan);
+               }
+       }
+       return res;
+}


 /* Hebrew syntax */
